package net.wanji.utc.service.common.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.Result;
import net.wanji.utc.mapper.CrossLightsMapper;
import net.wanji.utc.mapper.CrossPhaseLightsMapper;
import net.wanji.utc.mapper.CrossPhaseMapper;
import net.wanji.utc.mapper.CrossSchemeMapper;
import net.wanji.utc.po.CrossLightsPO;
import net.wanji.utc.po.CrossPhaseLightsPO;
import net.wanji.utc.po.CrossPhasePO;
import net.wanji.utc.po.CrossSchemePO;
import net.wanji.utc.service.common.WanjiCommonStaticInfoService;
import net.wanji.utc.util.HttpRestUtil;
import net.wanji.utc.vo.SchemePhaseLightsVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/09 9:21
 */
@Service
@Slf4j
public class WanjiCommonStaticInfoServiceImpl implements WanjiCommonStaticInfoService {

    @Autowired
    private HttpRestUtil httpRestUtil;
    @Value("url")
    private String rpcUri;
    @Autowired
    private CrossSchemeMapper crossSchemeMapper;
    @Autowired
    private CrossPhaseMapper crossPhaseMapper;
    @Autowired
    private CrossLightsMapper crossLightsMapper;
    @Autowired
    private CrossPhaseLightsMapper crossPhaseLightsMapper;

    @Override
    public Result<SchemePhaseLightsVO> schemePhaseLights(String crossId, String signalId) {
        Map<String, String> paramMap = new HashMap<>(2);
        paramMap.put("crossId", crossId);
        paramMap.put("signalId", signalId);
        Result<SchemePhaseLightsVO> result = rpcService(rpcUri, null, JSONObject.toJSONString(paramMap), SchemePhaseLightsVO.class);
        if (Objects.nonNull(result.getContent())) {
            SchemePhaseLightsVO content = result.getContent();
            List<CrossSchemePO> crossSchemeList = content.getCrossSchemeList();
            crossSchemeList.forEach(crossSchemePO -> {
                crossSchemeMapper.deleteOne(crossId, crossSchemePO.getSchemeNo());
                crossSchemeMapper.insertOne(crossSchemePO);
            });
            List<CrossPhasePO> crossPhaseList = content.getCrossPhaseList();
            crossPhaseList.forEach(crossPhasePO -> {
                crossPhaseMapper.deleteBatch(crossId, String.valueOf(crossPhasePO.getPlanId()));
                crossPhaseMapper.insertOne(crossPhasePO);
            });

            List<CrossLightsPO> crossLightsList = content.getCrossLightsList();
            crossLightsList.forEach(crossLightsPO -> {
                crossLightsMapper.deleteOne(crossId, crossLightsPO.getName());
                crossLightsMapper.insertOne(crossLightsPO);
            });
            List<CrossPhaseLightsPO> crossPhaseLightsPOList = content.getCrossPhaseLightsPOList();
            if (!CollectionUtils.isEmpty(crossPhaseLightsPOList)) {
                crossPhaseLightsMapper.deleteByCrossId(crossPhaseLightsPOList.get(0).getCrossId());
                crossPhaseLightsMapper.insertBatch(crossPhaseLightsPOList);
            }
        }
        return Result.success();
    }

    @Override
    public <T> Result<T> rpcService(String url, Map<String, String> headers, String param, Class<T> classType) {
        Result<T> result = httpRestUtil.doPost(url, headers, param, Result.class);
        if (Objects.isNull(result) || result.getState() != 200) {
            log.error("远程服务调用异常，url:".concat(rpcUri));
            return Result.error("获取万集标准信号机方案数据远程调用异常！");
        }
        return result;
    }
}
